@@ -65,11 +65,6 @@ int _singularity_runtime_mount_scratch(void) {
         return(0);
     }
 
-#ifndef SINGULARITY_NO_NEW_PRIVS
-    singularity_message(WARNING, "Not mounting scratch: host does not support PR_SET_NO_NEW_PRIVS\n");
-    return(0);
-#endif  
-
     singularity_message(DEBUG, "Checking SINGULARITY_WORKDIR from environment\n");
     if ( ( tmpdir_path = singularity_registry_get("WORKDIR") ) == NULL ) {
         if ( ( tmpdir_path = singularity_registry_get("SESSIONDIR") ) == NULL ) {
@@ -98,7 +93,7 @@ int _singularity_runtime_mount_scratch(void) {
             ABORT(255);
         }
 
-        if ( s_mkpath(full_sourcedir_path, 0750) < 0 ) {
+        if ( container_mkpath(full_sourcedir_path, 0750) < 0 ) {
              singularity_message(ERROR, "Could not create scratch working directory %s: %s\n", full_sourcedir_path, strerror(errno));
              ABORT(255);
         }
@@ -107,7 +102,7 @@ int _singularity_runtime_mount_scratch(void) {
             if ( singularity_registry_get("OVERLAYFS_ENABLED") != NULL ) {
                 singularity_priv_escalate();
                 singularity_message(DEBUG, "Creating scratch directory inside container\n");
-                r = s_mkpath(full_destdir_path, 0755);
+                r = container_mkpath(full_destdir_path, 0755);
                 singularity_priv_drop();
                 if ( r < 0 ) {
                     singularity_message(VERBOSE, "Skipping scratch directory mount, could not create dir inside container %s: %s\n", current, strerror(errno));
